@@ -66,7 +66,6 @@ contacts_chooser_add_cb (GtkWidget *button, ContactsData *data)
 void
 contacts_search_changed_cb (GtkWidget *widget, ContactsData *data)
 {
-	gtk_widget_grab_focus (data->ui->search_entry);
 	contacts_update_treeview (data);
 }
 
@@ -159,7 +158,7 @@ contacts_delete_cb (GtkWidget *source, ContactsData *data)
 
 		name = e_contact_get_const (contact, E_CONTACT_FULL_NAME);
 		if ((!name) || (g_utf8_strlen (name, 4) <= 0))
-			name = _("Unknown");
+			name = _("Unnamed");
 
 		contact_list = g_list_prepend (contact_list, e_contact_get (contact, E_CONTACT_UID));
 
@@ -202,9 +201,11 @@ contacts_delete_cb (GtkWidget *source, ContactsData *data)
 		default:
 			break;
 	}
+	g_list_foreach (contact_list, (GFunc) g_free, NULL);
 	g_list_free (contact_list);
 	gtk_widget_destroy (dialog);
 	contacts_set_widgets_sensitive (widgets);
+	g_list_free (widgets);
 }
 
 void
@@ -561,25 +562,42 @@ contacts_paste_cb (GtkWindow *main_window)
 void
 contacts_about_cb (GtkWidget *parent)
 {
-	gchar *authors[] = {"Chris Lord <chris@o-hand.com>", NULL};
+	const char *authors[] = {"Chris Lord <chris@openedhand.com>",
+			    "Ross Burton <ross@openedhand.com>",
+			    "Matthew Allum <mallum@openedhand.com>",
+			    "Thomas Wood <thomas@openedhand.com>",
+			    NULL};
+	const char *license = {
+	  N_(
+	     "Contacts is free software; you can redistribute it and/or modify "
+	     "it under the terms of the GNU General Public License as published by "
+	     "the Free Software Foundation; either version 2 of the License, or "
+	     "(at your option) any later version.\n\n"
+	     "Contacts is distributed in the hope that it will be useful, "
+	     "but WITHOUT ANY WARRANTY; without even the implied warranty of "
+	     "MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the "
+	     "GNU General Public License for more details.\n\n"
+	     "You should have received a copy of the GNU General Public License "
+	     "along with Contacts; if not, write to the Free Software Foundation, Inc., "
+	     "51 Franklin St, Fifth Floor, Boston, MA 0110-1301, USA"
+	     )
+	};
 	/* Translators: please translate this as your own name and optionally email
 	   like so: "Your Name <your@email.com>" */
-	const gchar *translator_credits = _("translator-credits");
-	GdkPixbuf *icon;
+	const char *translator_credits = _("translator-credits");
 
-	icon = gdk_pixbuf_new_from_file (DATADIR"/pixmaps/oh-contacts.png", NULL);
 	gtk_show_about_dialog (GTK_WINDOW (parent),
 		"name", GETTEXT_PACKAGE,
 		"version", VERSION,
 		"authors", authors,
-		"logo", icon,
+		"logo-icon-name", "contacts",
 		"website", "http://projects.o-hand.com/contacts/",
-		"copyright", "(c) 2006 OpenedHand Ltd",
+		"copyright", "\302\251 2006 OpenedHand Ltd",
 		"translator-credits", translator_credits,
+		"license", license,
+		"wrap-license", TRUE,
 		NULL);
 
-	if (icon != NULL)
-		g_object_unref (icon);
 }
 
 gboolean
@@ -587,7 +605,7 @@ contacts_treeview_search_cb (GtkWidget *search_entry, GdkEventKey *event,
 	GtkTreeView *treeview)
 {
 	gtk_widget_event (search_entry, (GdkEvent *)event);
-	gtk_entry_set_position (GTK_ENTRY (search_entry), -1);
+	gtk_editable_set_position (GTK_EDITABLE (search_entry), -1);
 	
 	return FALSE;
 }
@@ -599,7 +617,7 @@ contacts_is_row_visible_cb (GtkTreeModel * model, GtkTreeIter * iter,
 	gboolean result = FALSE;
 	gchar *group = NULL;
 	GList *groups, *g;
-	const gchar *uid;
+	gchar *uid;
 	EContactListHash *hash;
 	const gchar *search_string;
 	ContactsData *data;
@@ -608,10 +626,11 @@ contacts_is_row_visible_cb (GtkTreeModel * model, GtkTreeIter * iter,
 	gtk_tree_model_get (model, iter, CONTACT_UID_COL, &uid, -1);
 	if (!uid) return FALSE;
 	hash = g_hash_table_lookup (contacts_table, uid);
+	g_free (uid);
 	if (!hash || !hash->contact) return FALSE;
 	data = hash->contacts_data;
 
-	if (data->selected_group && strcmp ("All", data->selected_group))
+	if (data->selected_group && strcmp (_("All"), data->selected_group))
 	{
 		groups = e_contact_get (hash->contact, E_CONTACT_CATEGORY_LIST);
 		if ((group = data->selected_group)) {
@@ -711,11 +730,30 @@ contacts_sort_treeview_cb (GtkTreeModel * model, GtkTreeIter * a, GtkTreeIter *
 	gtk_tree_model_get (model, b, CONTACT_NAME_COL, &string2, -1);
 	if (!string1) string1 = g_new0 (gchar, 1);
 	if (!string2) string2 = g_new0 (gchar, 1);
-	returnval = strcasecmp ((const char *) string1,
-				(const char *) string2);
+
+	/* Necessary for case independant comparison */
+
+	string1 = g_utf8_casefold (string1, -1);
+	string2 = g_utf8_casefold (string2, -1);
+	returnval = g_utf8_collate ((const char *) string1,
+				    (const char *) string2);
 	g_free (string1);
 	g_free (string2);
 
 	return returnval;
 }
 
+gboolean
+contacts_main_window_delete_event_cb (GtkWidget *main_window, gpointer data)
+{
+#ifdef HAVE_GCONF
+	GConfClient *client;
+	gint width, height;
+	client = gconf_client_get_default ();
+	gtk_window_get_size (GTK_WINDOW (main_window), &width, &height);
+	gconf_client_set_int (client, GCONF_PATH "/width", width, NULL);
+	gconf_client_set_int (client, GCONF_PATH "/height", height, NULL);
+	g_object_unref (G_OBJECT (client));
+#endif
+	return FALSE;
+}
