@@ -130,7 +130,24 @@ contacts_edit_ok_cb (GtkWidget *button, ContactsData *data)
 		/* Commit changes */
 		e_book_commit_contact(data->book, data->contact, &error);
 		if (error) {
-			/* TODO: show dialog */
+			GtkWidget *dialog;
+			GtkWidget *toplevel = gtk_widget_get_toplevel (button);
+			dialog = gtk_message_dialog_new (
+					GTK_WINDOW (toplevel),
+					GTK_DIALOG_MODAL,
+					GTK_MESSAGE_ERROR,
+					GTK_BUTTONS_OK,
+					_("Couldn't update contact")
+					);
+
+			gtk_message_dialog_format_secondary_text (
+					GTK_MESSAGE_DIALOG (dialog),
+					_("Information couldn't be updated, error was: %s"), 
+					error->message 
+					);
+
+			gtk_dialog_run (GTK_DIALOG (dialog));
+			gtk_widget_destroy (dialog);		
 			g_warning ("Cannot commit contact: %s", error->message);
 			g_error_free (error);
 		}
@@ -153,7 +170,24 @@ contacts_edit_ok_new_cb (GtkWidget *button, ContactsData *data)
 			GError *error = NULL;
 			e_book_add_contact(data->book, data->contact, &error);
 			if (error) {
-				/* TODO: show dialog */
+				GtkWidget *dialog;
+				GtkWidget *toplevel = gtk_widget_get_toplevel (button);
+				dialog = gtk_message_dialog_new (
+						GTK_WINDOW (toplevel),
+						GTK_DIALOG_MODAL,
+						GTK_MESSAGE_ERROR,
+						GTK_BUTTONS_OK,
+						_("Couldn't add contact")
+						);
+
+				gtk_message_dialog_format_secondary_text (
+						GTK_MESSAGE_DIALOG (dialog),
+						_("Cannot add contact: %s"), 
+						error->message 
+						);
+
+				gtk_dialog_run (GTK_DIALOG (dialog));
+				gtk_widget_destroy (dialog);		
 				g_warning ("Cannot add contact: %s",
 					error->message);
 				g_error_free (error);
@@ -321,7 +355,7 @@ contacts_label_widget_new (EContact *contact, EVCardAttribute *attr,
 	gchar *label_markup;
 
 	/* Create label/button text */
-	label_markup = g_strdup_printf ("<b>%s:</b>", name);
+	label_markup = g_strdup_printf (_("<b>%s:</b>"), name);
 
 	/* Create widget */
 	label_widget = gtk_label_new (NULL);
@@ -427,8 +461,13 @@ contacts_edit_widget_new (EContact *contact, EVCardAttribute *attr,
 				multiline = sfield->multiline;				
 				gtk_label_set_use_markup (GTK_LABEL (label),
 							  TRUE);
+
 				label_markup = g_strdup_printf (
-					"<b>%s:</b>",
+                                        /** Translators, the first
+                                         * argument is the field's
+                                         * name, ex. <b>Country:</b>
+                                         */				
+                                        _("<b>%s:</b>"),
 					gettext(sfield->subfield_name));
 				gtk_label_set_markup (GTK_LABEL (label),
 						      label_markup);
@@ -498,7 +537,7 @@ contacts_edit_widget_new (EContact *contact, EVCardAttribute *attr,
 		}
 	} else if (multi_line) {
 		/* Handle single-valued fields that span multiple lines */
-		const gchar *string = e_vcard_attribute_get_value (attr);
+		gchar *string = e_vcard_attribute_get_value (attr);
 		
 		GtkWidget *container = NULL;
 		GtkTextView *view = GTK_TEXT_VIEW (gtk_text_view_new ());
@@ -507,7 +546,8 @@ contacts_edit_widget_new (EContact *contact, EVCardAttribute *attr,
 		gtk_widget_set_name (GTK_WIDGET (view), attr_name);
 		gtk_text_buffer_set_text (buffer, string ? string : "", -1);
 		gtk_text_view_set_editable (view, TRUE);
-		
+		g_free (string);
+
 		container = gtk_frame_new (NULL);
 		gtk_frame_set_shadow_type (
 			GTK_FRAME (container), GTK_SHADOW_IN);
@@ -533,13 +573,14 @@ contacts_edit_widget_new (EContact *contact, EVCardAttribute *attr,
 				  (contacts_entry_changed), data);
 	} else {
 		/* Handle simple single-valued single-line fields */
-		const gchar *string = e_vcard_attribute_get_value (attr);
+		gchar *string = e_vcard_attribute_get_value (attr);
 		GtkWidget *entry, *container = NULL;
 
 		entry = gtk_entry_new ();
 		gtk_entry_set_text (GTK_ENTRY (entry),
 				    string ? string : "");
 		gtk_widget_set_name (entry, attr_name);
+		g_free (string);
 
 /*		if (type_edit) {
 			gtk_widget_show (type_edit);
@@ -630,6 +671,7 @@ contacts_edit_add_focus_events (GtkWidget *widget, GtkWidget *ebox,
 			contacts_edit_add_focus_events (GTK_WIDGET (c->data),
 				ebox, widgets);
 		}
+		g_list_free (children);
 	} else if (GTK_IS_WIDGET (widget)) {	
 		GList *w;
 		g_signal_connect (G_OBJECT (widget), "focus-in-event",
@@ -713,7 +755,6 @@ contacts_append_to_edit_table (GtkTable *table, GtkWidget *label,
 
 typedef struct {
 	EVCardAttribute *attr;
-	GList *contacts_groups;
 	ContactsData *contacts_data;
 } ContactsGroupChangeData;
 
@@ -724,7 +765,7 @@ contacts_change_groups_cb (GtkWidget *widget, ContactsGroupChangeData *data)
 	GList *results = NULL;
 	GList *values = e_vcard_attribute_get_values (data->attr);
 
-	for (g = data->contacts_groups; g; g = g->next) {
+	for (g = data->contacts_data->contacts_groups; g; g = g->next) {
 		if (g_list_find_custom (values, g->data, (GCompareFunc)strcmp))
 			bools = g_list_append (bools, GINT_TO_POINTER (TRUE));
 		else
@@ -732,17 +773,19 @@ contacts_change_groups_cb (GtkWidget *widget, ContactsGroupChangeData *data)
 	}
 	
 	if (contacts_chooser (data->contacts_data, _("Change groups"), _("<b>Choose groups"
-		"</b>"), data->contacts_groups, bools, TRUE, &results)) {
-/*		gchar *new_groups = results ?
-			contacts_string_list_as_string (results, ", ") :
+		"</b>"), data->contacts_data->contacts_groups, bools, TRUE, TRUE, &results)) {
+
+		gchar *new_groups = results ?
+			contacts_string_list_as_string (results, ", ", FALSE) :
 			g_strdup ("None");
-		gtk_button_set_label (GTK_BUTTON (widget), new_groups);
-		g_free (new_groups);*/
+		g_free (new_groups);
+
 		e_vcard_attribute_remove_values (data->attr);
 		for (g = results; g; g = g->next) {
 			e_vcard_attribute_add_value (data->attr, g->data);
+			if (!g_list_find_custom (data->contacts_data->contacts_groups, g->data, (GCompareFunc) strcmp))
+					data->contacts_data->contacts_groups = g_list_prepend (data->contacts_data->contacts_groups, g_strdup (g->data));
 		}
-		g_list_free (results);
 		data->contacts_data->changed = TRUE;
 	}
 }
@@ -794,7 +837,7 @@ contacts_add_field_cb (GtkWidget *button, ContactsData *data)
 	if (contacts_chooser (data, _("Add field"),
                               /* TODO: make nicer for i18n */
 			      _("<b>Choose a field</b>"), fields,
-			      NULL, FALSE, &field)) {
+			      NULL, FALSE, FALSE, &field)) {
 		GtkWidget *label, *edit, *table;
 		const ContactsField *cfield;
 		EVCardAttribute *attr;
@@ -815,6 +858,7 @@ contacts_add_field_cb (GtkWidget *button, ContactsData *data)
 		contacts_append_to_edit_table (GTK_TABLE (table), label, edit,
 			TRUE);
 		
+		g_list_foreach (field, (GFunc) g_free, NULL);
 		g_list_free (field);
 	}
 	
@@ -1169,7 +1213,6 @@ contacts_edit_pane_show (ContactsData *data, gboolean new)
 	gdata = g_new (ContactsGroupChangeData, 1);
 	gdata->contacts_data = data;
 	gdata->attr = groups_attr;
-	gdata->contacts_groups = data->contacts_groups;
 	/* Remove any old signals and replace with new ones with the correct
 	 * user data.
 	 */
