@@ -90,7 +90,7 @@ contacts_ui_update_groups_list (ContactsData *data)
 	list = (GtkListStore*) gtk_combo_box_get_model (GTK_COMBO_BOX(groups_combobox));
 	if (!list) return;
 	gtk_list_store_clear (list);
-	gtk_list_store_insert_with_values (list, &dummy, 0, 0, "All", -1);
+	gtk_list_store_insert_with_values (list, &dummy, 0, 0, _("All"), -1);
 	g_list_foreach (data->contacts_groups, add_to_list, list);
 
 	/* Select 'All' in the groups combobox if nothing selected */
@@ -201,7 +201,7 @@ create_main_window (ContactsData *data)
 	gtk_container_add (GTK_CONTAINER (contacts_menu_menu), delete_menuitem);
 	gtk_widget_set_sensitive (delete_menuitem, FALSE);
 
-	contacts_import = gtk_menu_item_new_with_mnemonic (_("_Import"));
+	contacts_import = gtk_menu_item_new_with_mnemonic (_("_Import..."));
 	gtk_container_add (GTK_CONTAINER (contacts_menu_menu), contacts_import);
 
 	widget = gtk_separator_menu_item_new ();
