@@ -156,12 +156,15 @@ kozo_utf8_strcasestrip (const char *str)
 /* Note: PHOTO and CATEGORIES are special-cased (see contacts_edit_pane_show) */
 static const ContactsField contacts_fields[] = {
 	{ "FN", E_CONTACT_FULL_NAME, NULL, FALSE, 10, TRUE },
+	{ "ORG", E_CONTACT_ORG, NULL, FALSE, 15, TRUE },
 	{ "TEL", 0, N_("Phone"), FALSE, 20, FALSE },
 	{ "EMAIL", 0, N_("Email"), FALSE, 30, FALSE },
+	{ "X-JABBER", 0, N_("Jabber"), FALSE, 35, FALSE },
 	{ "ADR", 0, N_("Address"), FALSE, 40, FALSE },
 	{ "NICKNAME", E_CONTACT_NICKNAME, NULL, FALSE, 110, TRUE },
-	{ "URL", E_CONTACT_HOMEPAGE_URL, N_("Homepage"), FALSE, 120, FALSE },
-	{ "NOTE", E_CONTACT_NOTE, NULL, TRUE, 130, TRUE },
+	{ "BDAY", E_CONTACT_BIRTH_DATE, NULL, FALSE, 120, TRUE },
+	{ "URL", E_CONTACT_HOMEPAGE_URL, N_("Homepage"), FALSE, 130, FALSE },
+	{ "NOTE", E_CONTACT_NOTE, NULL, TRUE, 140, TRUE },
 	{ NULL }
 };
 
@@ -264,16 +267,37 @@ contacts_field_pretty_name (const ContactsField *field)
 		return NULL;
 }
 
+gint
+contacts_compare_attributes (EVCardAttribute *attrA, EVCardAttribute *attrB)
+{
+	gint a = 0, b = 0, i;
+	const gchar *nameA, *nameB;
+
+	nameA = e_vcard_attribute_get_name (attrA);
+	nameB = e_vcard_attribute_get_name (attrB);
+
+	for (i = 0; contacts_fields[i].vcard_field; i++)
+	{
+		if (strcmp (nameA, contacts_fields[i].vcard_field))
+			a = contacts_fields[i].priority;
+		if (strcmp (nameB, contacts_fields[i].vcard_field))
+			b = contacts_fields[i].priority;
+	}
+
+	return b - a;
+}
+
 EContact *
 contacts_contact_from_tree_path (GtkTreeModel *model, GtkTreePath *path, GHashTable *contacts_table)
 {
 	GtkTreeIter iter;
 	if (gtk_tree_model_get_iter (model, &iter, path)) {
-		const gchar *uid;
+		gchar *uid;
 		EContactListHash *hash;
 		gtk_tree_model_get (model, &iter, CONTACT_UID_COL, &uid, -1);
 		if (uid) {
 			hash = g_hash_table_lookup (contacts_table, uid);
+			g_free (uid);
 			if (hash)
 				return hash->contact;
 		}
@@ -467,7 +491,7 @@ gchar *
 contacts_string_list_as_string (GList *list, const gchar *separator,
 				gboolean include_empty)
 {
-	gchar *old_string, *new_string;
+	gchar *old_string, *new_string = NULL;
 	GList *c;
 	
 	if (!include_empty)
@@ -665,14 +689,20 @@ contacts_entries_get_values (GtkWidget *widget, GList *list) {
 	return list;
 }
 
+static void
+contacts_chooser_treeview_row_activated (GtkTreeView *tree, gpointer data)
+{
+	GtkWidget *dialog = g_object_get_data (G_OBJECT (tree), "parent_dialog");
+	gtk_dialog_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
+}
+
 gboolean
 contacts_chooser (ContactsData *data, const gchar *title, const gchar *label_markup,
-		  GList *choices, GList *chosen, gboolean allow_custom,
+		  GList *choices, GList *chosen, gboolean allow_custom,gboolean multiple_choice,
 		  GList **results)
 {
 	GList *c, *d, *widgets;
 	GtkWidget *label_widget;
-	gboolean multiple_choice = chosen ? TRUE : FALSE;
 	GtkWidget *dialog = data->ui->chooser_dialog;
 	GtkTreeView *tree = GTK_TREE_VIEW (data->ui->chooser_treeview);
 	GtkTreeModel *model = gtk_tree_view_get_model (tree);
@@ -726,6 +756,18 @@ contacts_chooser (ContactsData *data, const gchar *title, const gchar *label_mar
 	
 	widgets = contacts_set_widgets_desensitive (
 		data->ui->main_window);
+
+	/*
+	 * Set the parent dialog as data on the treeview.
+	 * Passing the dialog through the user data callback causes problems
+	 * because of the interaction between the application and the dialog
+	 * main loops.
+	 */
+	g_object_set_data (G_OBJECT (tree), "parent_dialog", dialog);
+
+	if (!multiple_choice)
+		g_signal_connect (G_OBJECT(tree), "row-activated", (GCallback) contacts_chooser_treeview_row_activated, NULL);
+
 	dialog_code = gtk_dialog_run (GTK_DIALOG (dialog));
 	gtk_widget_hide (dialog);
 	if (dialog_code == GTK_RESPONSE_OK) {
@@ -762,14 +804,15 @@ contacts_chooser (ContactsData *data, const gchar *title, const gchar *label_mar
 			} else {
 				gtk_tree_model_get (model, &iter,
 					CHOOSER_NAME_COL, &selection_name, -1);
-				*results = g_list_append (NULL, selection_name);				
+				*results = g_list_append (NULL, selection_name);
 				returnval = TRUE;
 			}
 		}
 	}
 	
 	contacts_set_widgets_sensitive (widgets);
-	
+	g_list_free (widgets);
+
 	return returnval;
 }
 
